c Test program for the GETMEM function.  This function ONLY works under
c DOS and not under Windows. Windows is a DPMI server and it provides
c different services then the Phar Lap VMM and DPMI interrupts must be
c used to get this information.
c
c Below is the interface statement to the function GETMEM.  This 
c interface statement must be the first line in any source file 
c that attempts to access the function.  The function takes an
c argument of type MEMINFO and of type BYTE (by value).  The 
c MEMINFO structure returns with detailed information about
c the memory configuration of the application.  The function itself
c returns just the maxmem element of the structure which is the largest
c block of available memory. The second argument takes a 1 to reset the
c memory statistics or 0 to not reset them. Most of the data is in the
c form of pages.  To get actual number of bytes, multipy by 4096. The
c function return does this already in the assembly code.

      interface to integer*4 function getmem[alias:'_GETMEM@8']
     +                                      (mem,reset)
      byte reset[value]
      structure /meminfo/
        logical*4  present    ! Is VMM present? (logical)
        integer*4  nconvpg    ! Total number of conventional memory pages      
        integer*4  reserved1  ! reserved  
        integer*4  nextpg     ! Total number of physical pages from all sources
        integer*4  extlim     ! Extended memory page limit
        integer*4  aphyspg    ! Total number of physical pages owned by application
        integer*4  alockpg    ! Total number of locked pages owned by appliation
        integer*4  sysphyspg  ! Total number of physical memory pages owned by application
        integer*4  reserved2  ! reserved 
        integer*4  begaddr    ! Linear address of beginning of application space
        integer*4  endaddr    ! Linear address of the end of application space
        integer*4  lasttime   ! Number of seconds since last memory statistic reset
        integer*4  numfault   ! Total number of page faults since last reset
        integer*4  numpage    ! Number of pages written to swap file since last reset
        integer*4  numpager   ! Number of reclaimed pages (pages swapped to disk and then back)
        integer*4  numowned   ! Number of code and data pages owned by the application
        integer*4  swapsize   ! Swap file size in pages
        integer*4  reserved3  ! reserved
        integer*4  minconpg   ! Minimum number of conventional memory pages
        integer*4  maxswap    ! Maximum number of pages swap file can grow to
        integer*4  pagefault  ! Page fault in progress (if so first bit is 1)
        integer*4  nfreepg    ! Number of free physical pages guaranteed by the VMM
        integer*4  navailpg   ! Number of physical pages currently available
        integer*4  maxmem     ! Largest available block of free memory in pages
        integer*4  padding    ! Padding out to 100 bytes       
      end structure
      record /meminfo/ mem[reference]
      end

      program testmem  

c This program asks you for input for an allocation size then outputs
c memory information prior to the allocation.  It does the allocation
c and then output memory data during initialization of the allocated 
c memory. When the VMM starts to swap out to disk the loop exits.
      
      structure /meminfo/
        logical*4  present      
        integer*4  nconvpg      
        integer*4  reserved1    
        integer*4  nextpg       
        integer*4  extlim       
        integer*4  aphyspg      
        integer*4  alockpg      
        integer*4  sysphyspg    
        integer*4  reserved2    
        integer*4  begaddr      
        integer*4  endaddr      
        integer*4  lasttime     
        integer*4  numfault     
        integer*4  numpage      
        integer*4  numpager     
        integer*4  numowned     
        integer*4  swapsize     
        integer*4  reserved3    
        integer*4  minconpg     
        integer*4  maxswap      
        integer*4  pagefault    
        integer*4  nfreepg      
        integer*4  navailpg     
        integer*4  maxmem       
        integer*4  padding         
      end structure
                   
      record /meminfo/ mem                                       
      integer*4 getmem
      integer*1 arr[allocatable](:)

      print"(1x,a\)", 'Input allocation size: '
      read(*,*) n
      print*, 'Memory stats prior to allocation:'
      call outputmem
      print"(/a)", '       ***** Press enter to continue *****'
      read*

      allocate (arr(n))

      print*, 'Memory stats after allocation: '
      call outputmem
      print"(/a)", '       ***** Press enter to continue *****'
      read*
      print*, 'Initializing allocated memory'
      do i=1,n
        arr(i) = 77
        if (mod(i,100000).eq.0) then
          k = getmem(mem,int1(0))
          print"('+',i10,' pages used at iteration ',i10)", 
     +           mem.numpage,i       
          if (mem.numpage.gt.0) then
            print*, 'Paging to swap file started...exiting loop'
            exit
          end if
        end if
      end do
      print*, 'Memory statistics following initialization'
      call outputmem
      end

c Subroutine to output the contents of the MEMINFO structure.            

      subroutine outputmem
      
      structure /meminfo/
        logical*4  present      
        integer*4  nconvpg      
        integer*4  reserved1    
        integer*4  nextpg       
        integer*4  extlim       
        integer*4  aphyspg      
        integer*4  alockpg      
        integer*4  sysphyspg    
        integer*4  reserved2    
        integer*4  begaddr      
        integer*4  endaddr      
        integer*4  lasttime     
        integer*4  numfault     
        integer*4  numpage      
        integer*4  numpager     
        integer*4  numowned     
        integer*4  swapsize     
        integer*4  reserved3    
        integer*4  minconpg     
        integer*4  maxswap      
        integer*4  pagefault    
        integer*4  nfreepg      
        integer*4  navailpg     
        integer*4  maxmem       
        integer*4  padding         
      end structure
                   
      record /meminfo/ mem                                       
      integer*4 getmem
      
      print*
      print*, 'Largest free block of memory:     ',getmem(mem,int1(0))
      print*, 'VMM present:                      ', mem.present
      print*, 'Conventional memory available:    ', mem.nconvpg * 4096
      print*, 'Total all types of phys. memory:  ', mem.nextpg * 4096
      print*, 'Extended page limit:              ', mem.extlim
      print*, 'Total owned pages:                ', mem.aphyspg
      print*, 'Total locked pages:               ', mem.alockpg
      print*, 'Total used by extender:           ', mem.sysphyspg * 4096
      print*, 'Pages written to swapfile:        ', mem.numpage
      print*, 'Number of reclaimed pages:        ', mem.numpager
      print*, 'Code and data owned by program:   ', mem.numowned * 4096
      print*, 'Size of VM swapfile:              ', mem.swapsize * 4096
      print*, 'Maximum swapfile pages:           ', mem.maxswap
      print*, 'Free guaranteed physical memory:  ', mem.nfreepg * 4096
      print*, 'Free available physical memory:   ', mem.navailpg * 4096
      print*, 'Largest free block of memory:     ', mem.maxmem * 4096

      end
